function [A,B,ys,info] = dynare_resolve(iv,ic,aux)

% function [A,B,ys,info] = dynare_resolve(iv,ic,aux)
% Computes the linear approximation and the matrices A and B of the
% transition equation
%
% INPUTS
%    iv:             selected variables (observed and state variables)
%    ic:             state variables position in the transition matrix columns
%    aux:            indices for auxiliary equations
%
% OUTPUTS
%    A:              matrix of predetermined variables effects in linear solution (ghx)
%    B:              matrix of shocks effects in linear solution (ghu)
%    ys:             steady state of original endogenous variables
%    info=1:         the model doesn't determine the current variables '...' uniquely
%    info=2:         MJDGGES returns the following error code'
%    info=3:         Blanchard Kahn conditions are not satisfied: no stable '...' equilibrium
%    info=4:         Blanchard Kahn conditions are not satisfied:'...' indeterminacy
%    info=5:         Blanchard Kahn conditions are not satisfied:'...' indeterminacy due to rank failure
%    info=20:        can't find steady state info(2) contains sum of sqare residuals
%    info=30:        variance can't be computed
%
% SPECIAL REQUIREMENTS
%    none

% Copyright (C) 2003-2007 Dynare Team
%
% This file is part of Dynare.
%
% Dynare is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% Dynare is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with Dynare.  If not, see <http://www.gnu.org/licenses/>.

  global oo_ M_
 
NumberOfParameters = M_.param_nbr;                            % Number of deep parameters.
for i = 1:NumberOfParameters                                  % Loop...
  paramname = deblank(M_.param_names(i,:));                   %    Get the name of parameter i. 
  eval([ paramname ' = M_.params(' int2str(i) ');']);         %    Get the value of parameter i.
end  

sig_r=0.0;
sig_rc=0.0;
cov_mc=-0.005;
sig_m=0.3;
sig_rq=0;
SR=0; 
RB40=0;
% NumberOfParameters = M_.param_nbr;
%         for i = 1:NumberOfParameters
%             paramname = deblank(M_.param_names(i,:));
%             eval(['M_.params(' int2str(i) ')=' paramname ';']);
%         end

%  [delta_tilde, nu, A, betta, allpha, kappa, rc_bar, k_bar, y_bar, c_bar, x_bar, q_bar, n_bar, m_bar, w_bar, ...
%    wf_bar, lambda_bar, H_bar, F_bar, ADJ_bar, ADJFOC_bar, ld_bar, cd_bar, g_bar ]=solve_ss_model1;
  

NumberOfParameters = M_.param_nbr;
        for i = 1:NumberOfParameters
            paramname = deblank(M_.param_names(i,:));
            eval(['M_.params(' int2str(i) ')=' paramname ';']);
        end
  
  %[oo_.steady_state,tchek] = feval([M_.fname '_steadystate'],[],[]);     
  [oo_.dr,info] = resol(oo_.steady_state,0);
  
    %% check for second moments
  
   if info(1) > 0
    A = [];
    B = [];
    ys = [];
    return
  end
 

nn=0;
% disp('--------------------------------------------------')
% disp(' ')
% disp('Starting iteration to resolve fixed-point problem:')
% disp(' ')
% disp('SS and paramters before iteration:')
% disp(' ')
% disp(sprintf('Rc: %0.5g', rc_bar));
% disp(sprintf('r: %0.5g', r_bar));
% disp(sprintf('sig_m: %0.5g', sig_m));
% disp(sprintf('cov_mc: %0.5g', cov_mc));
% disp(' ')

[sig_m_new cov_mc_new sig_rc_new sig_r_new  SR_new sig_rq_new RB40_new]=get_secondmoments(M_.Sigma_e);


while  abs(sig_m_new - sig_m) >1e-6 || abs(cov_mc_new -cov_mc)>1e-6 
 
  nn=nn+1;  
  
    
 sig_m=sig_m_new;
 cov_mc=cov_mc_new;
 sig_rc=sig_rc_new;
 sig_r=sig_r_new;
 sig_rq=sig_rq_new;   
 SR=SR_new;
 RB40=RB40_new;
 
%     NumberOfParameters = M_.param_nbr;
%         for i = 1:NumberOfParameters
%             paramname = deblank(M_.param_names(i,:));
%             eval(['M_.params(' int2str(i) ')=' paramname ';']);
%         end
%     
%   [delta_tilde, nu, A, betta, allpha, kappa, rc_bar, k_bar, y_bar, c_bar, x_bar, q_bar, n_bar, m_bar, w_bar, ...
%    wf_bar, lambda_bar, H_bar, F_bar, ADJ_bar, ADJFOC_bar, ld_bar, cd_bar, g_bar ]=solve_ss_model1;

  %P=-cov_mc/sig_rc*sig_rq;

    NumberOfParameters = M_.param_nbr;
        for i = 1:NumberOfParameters
            paramname = deblank(M_.param_names(i,:));
            eval(['M_.params(' int2str(i) ')=' paramname ';']);
        end
   %[oo_.steady_state,tchek] = feval([M_.fname '_steadystate'],[],[]);
  
    [oo_.dr, info] = resol(oo_.steady_state,0);
    if nn==1000
        disp('hit')
      info(1)=10; 
    end  

   if info(1) > 0
    A = [];
    B = [];
    ys = [];
    return
  end
    
[sig_m_new cov_mc_new sig_rc_new sig_r_new  SR_new sig_rq_new RB40_new]=get_secondmoments(M_.Sigma_e);

end  

sig_m=sig_m_new;
 cov_mc=cov_mc_new;
 sig_rc=sig_rc_new;
 sig_r=sig_r_new;
 sig_rq=sig_rq_new;   
 SR=SR_new;
 RB40=RB40_new;
 
  NumberOfParameters = M_.param_nbr;
        for i = 1:NumberOfParameters
            paramname = deblank(M_.param_names(i,:));
            eval(['M_.params(' int2str(i) ')=' paramname ';']);
        end

  if info(1) > 0
    A = [];
    B = [];
    ys = [];
    return
  end
  
  if nargin == 0
    endo_nbr = M_.endo_nbr;
    nstatic = oo_.dr.nstatic;
    npred = oo_.dr.npred;
    iv = (1:endo_nbr)';
    ic = [ nstatic+(1:npred) endo_nbr+(1:size(oo_.dr.ghx,2)-npred) ]';
    aux = oo_.dr.transition_auxiliary_variables;
    k = find(aux(:,2) > npred);
    aux(:,2) = aux(:,2) + nstatic;
    aux(k,2) = aux(k,2) + oo_.dr.nfwrd;
  end
  
  [A,B] = kalman_transition_matrix(oo_.dr,iv,ic,aux,M_.exo_nbr);
  ys = oo_.dr.ys;